function output=ECHAIM_Chars(lat, lon, alt, dbFile, varargin)
%Function to use the output from A-CHAIM and generate the
%background (E-CHAIM) ionospheric characteristics for the 
%input latitude, longitude, and altitude
%
%Inputs
%lat = array of latitudes
%lon = array of latitudes
%alt = array of latitudes (km)
%dbFile = the full or relative path to the input DB file (obtained from the
%   A-CHAIM project
%
%Options
%GRID - will generate a grid based on the input latitude and longitude
%   arrays, and output the characteristics for this grid.
%TEC - will output the total electron content instead of the electron
%   density

if ~exist(dbFile, 'file')
    error("ACHAIMCHARS:DBFILEDNE","The provided DB file does not exist.");
end

params = DB2params(dbFile);

%set the grid keyword based in user options
grid = 0;
tec = 0;
if sum(strcmpi(varargin,'grid')); grid = 1; end
if sum(strcmpi(varargin,'tec')); tec = 1; end

%if grid is chosen, check for equal lat/lon sizes
if ~grid && length(lat) ~= length(lon)
    error("ECHAIMCHARS:LATLONLENGTH","When using GRID, the latitude and longitude arrays must be the same length");
end

%create the lat/lon grid
if grid; [lat,lon] = ndgrid(lat,lon); end

%if tec is requested, create the alt array
if tec; alt = [85:15:550 650:100:2050]; end

%define ouput array size
si = [size(lat,1) size(lon,2)];

%define output
fields=[{'lat'};{'lon'};{'alt'};{'date'};{'hmE'}];
data=[{lat};{lon};{alt};{params(1).Time};{zeros(size(lat))+102}];
    
capList = cell2mat(cellfun(@(n,m) [n(:)',m(:)'],{params.Cap},{params.Rot},'un',0)');
NSph = max([[params.NSph],[params.NSphModel]]);

[~,j,i]=unique(capList,'rows');
Basis = cellfun(@(n,m) sph_cap_coords(lat(:),lon(:),NSph,'cap',n,'rot',m),{params(j).Cap},{params(j).Rot},'UniformOutput',false);
Basis=Basis(i);

%Calculate the characteristics
k = 1:length(params);
data = [data; arrayfun(@(x) reshape((Basis{x}(:,1:numel(params(x).ModelParameters))*params(x).ModelParameters),si),k,'un',0)'];
fields = [fields;{params(k).Name}'];

%prop
%add
%logabs
%abs

output = cell2struct(data,fields);

%Calculate the density

%secondary parameters
r = 20;
g = 0.18;
k = alt - output.hmF2(:);
Ht = output.HTop(:).*(1 + ((r*g*k)./((r*output.HTop(:))+(g*k))));
z = k./Ht;

%botomside parameters
dHBot = 1;
dHF1 = sech((k) ./ (0.5 * (output.hmF2(:) - output.hmF1(:)))).^2;
dHE  = sech((alt - output.hmE(1)) ./ 30.0).^2;

H = (output.HBot(:) .* dHBot + output.HF1(:) .* dHF1 + output.HE(:) .* dHE) .* (1 ./ (1 + exp((output.hmE(1)-15-alt) ./ 2.5)));
Nb = output.NmF2(:) .* (sech(k ./ H).^2);
Nt = output.NmF2(:) .* sech(z ./ 2).^2;

topSide = alt >= output.hmF2(:);

Nb(topSide) = 0;
Nt(~topSide) = 0;

%dimensions = [location pairs (2 dimensions possible), altitudes]
dens = reshape(Nb + Nt,[si numel(output.alt)]);

%set the final output array
if tec
    tecOutput = trapz(alt,dens,3) / 1e13;
    data = [data;{tecOutput}];
    fields = [fields;{'tec'}];
else
    data = [data;{dens}];
    fields = [fields;{'dens'}];
end

output = cell2struct(data,fields);

%remove bad data
n.HF1(isnan(output.hmF1) | output.hmF1 < 1) = 0;
output.hmF1(isnan(output.hmF1)) = 0;

%remove data outside the assimilation grid
inGrid = in_grid(params(1).Rot,45,output.lat,output.lon)

%if some inputs are outside the grid then set them to NaN
if sum(inGrid) < length(inGrid)
   output.hmE(~inGrid) = NaN; 
   output.NmF2(~inGrid) = NaN; 
   output.hmF2(~inGrid) = NaN; 
   output.HBot(~inGrid) = NaN; 
   output.HTop(~inGrid) = NaN; 
   output.HF1(~inGrid) = NaN; 
   output.HE(~inGrid) = NaN; 
   output.hmF1(~inGrid) = NaN; 
   if tec; output.tec(~inGrid) = NaN; else output.dens(~inGrid) = NaN; end
end

end

%Calculate spherical cap coodrinates
function Out = sph_cap_coords(lat,lon,lMax,varargin)

%define variables
cap = -90;
grid = false;
rot = false;

%check and deal with variable arguments
if sum(strcmpi(varargin,'GRID')); grid = true; end
if sum(strcmpi(varargin,'CAP')); cap = varargin{find(strcmpi(varargin,'cap'))+1}; end
if sum(strcmpi(varargin,'ROT')); rot = true; rotm = cell2mat(varargin(find(strcmpi(varargin,'rot'))+1)); end

%check for lat/lon dimensions
if numel(lat) ~= numel(lon) && ~grid
  error('SPHCAPCOORDS:LATLONSIZE','Lat and Lon must be the same size.');
end

%convert to radians
lat = deg2rad(lat);
lon = deg2rad(lon);
cap = deg2rad(cap);

%create location grid if asked for
if grid
  [lat,lon] = ndgrid(lat,lon);
end

%Rotate the latitude and longitude and convert to spherical
Sph=cart2sph(cart2sph([ones(numel(lat),1),lat(:),lon(:)],-1)*rotm,1);
lat = Sph(:,2);
lon = Sph(:,3);

%co-latitude
coLat = pi.*((pi/2)-lat(:))./((pi/2)-cap);

N = sum(2*(0:lMax))+(lMax+1);
Out = zeros(numel(lon),N);

Nlm = @(ll,mm) ((-1).^mm).*sqrt((2 - (mm==0)).*(2*ll + 1));

I = 1;
for L = 0:lMax
  Pl = legendre(L,cos(coLat),'sch');
  for m = 0:L
    P = Pl(m+1,:);
    P = P.*Nlm(L,m);
    
    Out(:,I) = (P(:).*cos(m*lon(:)))';
    I=I+1;
    
    if m == 0; continue; end
    
    Out(:,I) = (P(:).*sin(m*lon(:)))';
    I=I+1;
  end
end
end
